package com.qmth.teachcloud.common.service;

import com.qmth.teachcloud.common.bean.auth.AuthBean;
import com.qmth.teachcloud.common.bean.result.MenuResult;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.PrivilegePropertyEnum;

import java.util.List;
import java.util.Set;

/**
 * @Description: 缓存操作service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
public interface CommonCacheService {

    /**
     * 添加角色缓存
     *
     * @param roleId
     * @return
     */
    public SysRole roleCache(Long roleId);

    /**
     * 修改角色缓存
     *
     * @param roleId
     * @return
     */
    public SysRole updateRoleCache(Long roleId);

    /**
     * 删除角色缓存
     *
     * @param roleId
     */
    public void removeRoleCache(Long roleId);

    /**
     * 添加用户菜单缓存
     *
     * @param userId
     * @return
     */
    public MenuResult userMenuCache(Long userId);

    /**
     * 修改用户菜单缓存
     *
     * @param userId
     * @return
     */
    public MenuResult updateUserMenuCache(Long userId);

    /**
     * 删除用户菜单缓存
     *
     * @param userId
     */
    public void removeUserMenuCache(Long userId);

    /**
     * 添加用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser userCache(Long userId);

    /**
     * 修改用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser updateUserCache(Long userId);

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    public void removeUserCache(Long userId);

    /**
     * 添加用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean userAuthCache(Long userId);

    /**
     * 修改用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean updateUserAuthCache(Long userId);

    /**
     * 删除用户鉴权缓存
     *
     * @param userId
     */
    public void removeUserAuthCache(Long userId);

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    public BasicSchool schoolCache(Long schoolId);

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    public BasicSchool schoolCache(String code);

    /**
     * 添加机构缓存
     *
     * @param orgId
     * @return
     */
    public SysOrg orgCache(Long orgId);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 修改鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> updatePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);


    /**
     * 删除鉴权url
     *
     * @param privilegePropertyEnum
     */
    public void removePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 添加角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> rolePrivilegeCache(Long roleId);

    /**
     * 修改角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> updateRolePrivilegeCache(Long roleId);

    /**
     * 删除角色权限缓存
     *
     * @param roleId
     */
    public void removeRolePrivilegeCache(Long roleId);

    /**
     * 添加用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> userRolePrivilegeCache(Long userId);

    /**
     * 修改用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> updateUserRolePrivilegeCache(Long userId);

    /**
     * 删除用户角色权限缓存
     *
     * @param userId
     */
    public void removeUserRolePrivilegeCache(Long userId);

    /**
     * 删除考生个人报告缓存
     */
    public void removeExamStudentReportCache();

    /**
     * 删除院长缓存
     */
    public void removeCollegeDeanCache();

    /**
     * 删除学院学科缓存
     */
    public void removeCollegeCourseCache();

    /**
     * 删除开课课程考试总览图表缓存
     */
    public void removeSurveyTeacherViewCache();

    /**
     * 删除考生个人报告缓存
     *
     * @param schoolId
     * @param studentCode
     * @param examId
     * @param courseCode
     */
    public void removeExamStudentReportCache(Long schoolId, String studentCode, String examId, String courseCode);

    /**
     * 删除院长缓存
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param collegeId
     */
    public void removeCollegeDeanCache(Long schoolId, String semester, Long examId, Long collegeId);

    /**
     * 删除学院学科缓存
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     */
    public void removeCollegeCourseCache(Long schoolId, String semester, Long examId, String courseCode, Long collegeId);

    /**
     * 删除开课课程考试总览图表缓存
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @param collegeId
     */
    public void removeSurveyTeacherViewCache(Long schoolId, String semester, Long examId, String courseCode, Long collegeId);

}
