package com.qmth.teachcloud.common.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.dto.BlurryUserDto;
import com.qmth.teachcloud.common.bean.dto.LoginDto;
import com.qmth.teachcloud.common.bean.dto.UserDto;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.bean.params.UserSaveParams;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.FlowApproveNameEnum;
import org.springframework.util.LinkedMultiValueMap;

import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 用户表 服务类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
public interface SysUserService extends IService<SysUser> {

    IPage<UserDto> list(String loginName, String roleId, Boolean enable, String realName, Integer pageNumber, Integer pageSize);

    boolean saveUser(UserSaveParams userSaveParams);

    boolean enable(SysUser user) throws NoSuchAlgorithmException;

    boolean resetPassword(Long id) throws NoSuchAlgorithmException;

    boolean updatePassword(SysUser user) throws NoSuchAlgorithmException;

    boolean bindRoles(SysUser sysUser);

    LoginDto getUserRoles(SysUser sysUser);

    List<BlurryUserDto> listUser(String courseCode, String param);

    List<SysUser> listByOrgId(Long id);

    boolean saveCustomer(UserSaveParams userSaveParams);

    IPage<UserDto> listCustomer(String realName, Boolean enable, Integer pageNumber, Integer pageSize);

    /**
     * 根据外键查找教师信息
     *
     * @param schoolId    学校id
     * @param teacherName 教师名称
     * @return 学院数据
     */
    SysUser findByForeignKey(Long schoolId, String teacherName);

    /**
     * 临时-不鉴权保存用户 （默认创建用户角色为该学校管理员）
     *
     * @param userSaveParams 用户参数
     * @return 结果
     */
    boolean saveUserNoAuth(UserSaveParams userSaveParams);

    /**
     * 过滤拥有审核权限的人
     *
     * @param schoolId
     * @param orgIds
     * @param filter
     * @param approveName
     * @return
     */
    List<ApproveUserResult> filterFlowPrivilege(Long schoolId, Set<Long> orgIds, Boolean filter, FlowApproveNameEnum approveName);

    /**
     * 执行批量导入用户逻辑
     *
     * @param finalList 基础课程数据集合
     * @param map       参数
     * @return 结果
     */
    Map<String, Object> executeSysUserImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws NoSuchAlgorithmException;

    /**
     * 获取命题审批人
     *
     * @param courseCode
     * @return
     */
    public List<ApproveUserResult> findByAssignApprove(String courseCode);

    /**
     * 获取教研室主任审批人
     *
     * @param courseCode
     * @return
     */
    public List<ApproveUserResult> findByDirectorApprove(String courseCode);

    /**
     * 获取院长审批人
     *
     * @param orgIds
     * @return
     */
    public List<ApproveUserResult> findByPresidentApprove(List<Long> orgIds);

    /**
     * 根据id获取
     *
     * @param id
     * @return
     */
    public List<ApproveUserResult> findById(Long id);
}
