package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.CourseInfoDto;
import com.qmth.teachcloud.common.bean.dto.excel.BasicCourseImportDto;
import com.qmth.teachcloud.common.bean.dto.excel.DescribeImportDto;
import com.qmth.teachcloud.common.bean.params.BasicCourseParams;
import com.qmth.teachcloud.common.bean.result.BasicCourseResult;
import com.qmth.teachcloud.common.bean.result.ClazzDatasourceResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.OrgTypeEnum;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.checkerframework.checker.units.qual.C;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Resource
    private BasicCourseMapper basicCourseMapper;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private BasicUserCourseService basicUserCourseService;
    @Resource
    private BasicClazzService basicClazzService;


    @Override
    public List<BasicCourse> listCoursesByUserId(Long userId) {
        return this.baseMapper.listCoursesByUserId(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verifyCourseInfo(Long schoolId, String courseCode, String courseName, Long userId) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode);

        BasicCourse tmp = this.getOne(queryWrapper);
        if (tmp != null) {
            if (!courseName.equals(tmp.getName())) {
                throw ExceptionResultEnum.ERROR
                        .exception("课程代码和课程名称不匹配,课程代码为 ：'" + courseCode + "',输入的课程名称 ：'" + courseName + "',实际该代码对应的课程名称 ：'" + tmp.getName() + "'。");
            }
        } else {
            BasicCourse basicCourse = new BasicCourse();
            basicCourse.setName(courseName);
            basicCourse.setCode(courseCode);
            basicCourse.setSchoolId(schoolId);
            basicCourse.setId(SystemConstant.getDbUuid());
            basicCourse.setCreateId(userId);
            basicCourse.setCreateTime(System.currentTimeMillis());
            this.saveOrUpdate(basicCourse);
        }
    }

    @Override
    public List<CourseInfoDto> findByUserLoginNameAndRealName(String loginName, String realName) {
        System.out.println("---");
        if (loginName == null) {
            loginName = "";
        }
        if (realName == null) {
            realName = "";
        }
        return basicCourseMapper.findByUserLoginNameAndRealName(loginName, realName);
    }

    @Override
    public BasicCourse findByCourseCode(String courseCode) {
        return this.getOne(new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getCode, courseCode));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createCourse(Long schoolId, String courseCode, String courseName) {
        // 向'basic_course'表新增数据 - 教研分析用
        List<BasicCourse> basicCourseList = this.list(new QueryWrapper<BasicCourse>().lambda()
                .eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode));
        if (basicCourseList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("已存在学校id为【" + schoolId + "】,课程编号为【" + courseCode + "】的课程");
        }
        BasicCourse basicCourse = new BasicCourse();
        basicCourse.setId(SystemConstant.getDbUuid());
        basicCourse.setCode(courseCode);
        basicCourse.setName(courseName);
        basicCourse.setSchoolId(schoolId);
        this.save(basicCourse);
    }

    @Override
    public IPage<BasicCourseResult> basicCoursePage(String courseName, Long startCreateTime, Long endCreateTime, int pageNumber, int pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        courseName = SystemConstant.translateSpecificSign(courseName);
        IPage<BasicCourseResult> iPage = basicCourseMapper.findBasicCoursePage(new Page<>(pageNumber, pageSize), courseName, startCreateTime, endCreateTime, schoolId, orgIds);
        List<BasicCourseResult> list = iPage.getRecords();
        for (BasicCourseResult basicCourseResult : list) {
            String clazzIds = basicCourseResult.getClazzIds();
            String[] clazzIdArr = clazzIds.split(",");
            List<ClazzDatasourceResult> clazzDatasourceResultList = new ArrayList<>();
            for (String id : clazzIdArr) {
                ClazzDatasourceResult clazzDatasourceResult = new ClazzDatasourceResult();
                BasicClazz clazz = basicClazzService.getById(SystemConstant.convertIdToLong(id));
                clazzDatasourceResult.setId(clazz.getId());
                clazzDatasourceResult.setCode(clazz.getClazzCode());
                clazzDatasourceResult.setName(clazz.getClazzName());
                clazzDatasourceResultList.add(clazzDatasourceResult);
            }
            basicCourseResult.setClazzList(clazzDatasourceResultList);
        }
        return iPage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveBasicCourse(BasicCourseParams basicCourseParams, SysUser sysUser) {
        List<BasicCourseParams> basicCourseParamsList = new ArrayList<>();
        basicCourseParamsList.add(basicCourseParams);
        BasicCourse editBasicCourse = this.editEntityHelp(basicCourseParamsList, sysUser).get(0);
        this.saveOrUpdate(editBasicCourse);
        return editBasicCourse.getId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicCourseBatch(List<Long> idList) {
        if (idList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("请选择要删除的目标");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        basicUserCourseService.remove(new QueryWrapper<BasicUserCourse>().lambda().in(BasicUserCourse::getCourseId, idList).eq(BasicUserCourse::getSchoolId, sysUser.getSchoolId()));

        UpdateWrapper<BasicCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda()
                .set(BasicCourse::getEnable, false)
                .in(BasicCourse::getId, idList)
                .set(BasicCourse::getOrgId, sysUser.getOrgId())
                .set(BasicCourse::getUpdateId, sysUser.getId());

        return this.update(updateWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeBasicCourseImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) {
        // 班级分割标志
        final String clazzSplit = ",";

        SysUser sysUser = (SysUser) map.get(SystemConstant.SYS_USER);
        Long schoolId = sysUser.getSchoolId();
        List<BasicCourseParams> addBasicCourseParamsList = new ArrayList<>();

        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> basicCourseImportDtoList = excelMap.get(i);
            assert basicCourseImportDtoList != null;
            if (basicCourseImportDtoList.get(0) instanceof DescribeImportDto) {
                continue;
            }
            map.put("dataCount", basicCourseImportDtoList.size());
            for (int y = 0; y < Objects.requireNonNull(basicCourseImportDtoList).size(); y++) {
                BasicCourseImportDto basicCourseImportDto = (BasicCourseImportDto) basicCourseImportDtoList.get(y);
                String courseName = basicCourseImportDto.getCourseName();
                String courseCode = basicCourseImportDto.getCourseCode();
                String teachingRoomName = basicCourseImportDto.getTeachingRoomName();
                String clazz = basicCourseImportDto.getClazz();

                SysOrg sysOrg = sysOrgService.getOne(new QueryWrapper<SysOrg>().lambda().eq(SysOrg::getSchoolId, schoolId).eq(SysOrg::getEnable, true).eq(SysOrg::getType, OrgTypeEnum.TEACHING_ROOM).eq(SysOrg::getName, teachingRoomName));
                if (Objects.isNull(sysOrg)) {
                    throw ExceptionResultEnum.ERROR.exception("输入的教研室【" + teachingRoomName + "】不存在");
                }
                Long teachingRoomId = sysOrg.getId();
                String[] clazzList = clazz.split(clazzSplit);
                List<Long> clazzIdList = new ArrayList<>();
                for (String clzName : clazzList) {
                    BasicClazz basicClazz = basicClazzService.getOne(new QueryWrapper<BasicClazz>().lambda()
                            .eq(BasicClazz::getEnable, true)
                            .eq(BasicClazz::getSchoolId, schoolId)
                            .eq(BasicClazz::getClazzName, clzName));
                    if (Objects.isNull(basicClazz)) {
                        throw ExceptionResultEnum.ERROR.exception("所选择【班级】" + clzName + "不存在");
                    }
                    clazzIdList.add(basicClazz.getId());
                }

                BasicCourseParams basicCourseParams = new BasicCourseParams();
                basicCourseParams.setCourseCode(courseCode);
                basicCourseParams.setCourseName(courseName);
                basicCourseParams.setTeachingRoomId(teachingRoomId);
                basicCourseParams.setClazzIdList(clazzIdList);
                BasicCourse old = this.getOne(new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getEnable, true).eq(BasicCourse::getCode, courseCode));
                if (Objects.nonNull(old)) {
                    basicCourseParams.setId(old.getId());
                }
                addBasicCourseParamsList.add(basicCourseParams);
            }
            List<BasicCourse> editBasicCourse = this.editEntityHelp(addBasicCourseParamsList, sysUser);
            this.saveOrUpdateBatch(editBasicCourse);
        }
        return map;
    }

    @Override
    public Long getOrgIdBySchoolIdAndCourseCode(Long schoolId, String courseCode) {
        if (Objects.isNull(schoolId)) {
            schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        }
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode);
        BasicCourse basicCourse = this.getOne(queryWrapper);
        if (Objects.isNull(basicCourse)) {
            throw ExceptionResultEnum.ERROR.exception("课程不存在");
        }

        return basicCourse.getTeachingRoomId();
    }

    private List<BasicCourse> editEntityHelp(List<BasicCourseParams> basicCourseParamsList, SysUser sysUser) {
        Long schoolId = sysUser.getSchoolId();
        List<BasicCourse> basicCourseList = new ArrayList<>();
        for (BasicCourseParams basicCourseParams : basicCourseParamsList) {
            Long id = basicCourseParams.getId();
            String courseName = basicCourseParams.getCourseName();
            // 校验课程编号
            String courseCode = basicCourseParams.getCourseCode();
            BasicCourse checkCode = this.getOne(new QueryWrapper<BasicCourse>().lambda()
                    .eq(BasicCourse::getCode, courseCode)
                    .eq(BasicCourse::getSchoolId, schoolId)
                    .eq(BasicCourse::getEnable, true));
            if (Objects.nonNull(checkCode)) {
                if (!checkCode.getId().equals(id)) {
                    throw ExceptionResultEnum.ERROR.exception("课程编号【" + courseCode + "】重复");
                }
            }
            // 校验教研室id
            Long teachingRoomId = basicCourseParams.getTeachingRoomId();
            if (Objects.isNull(sysOrgService.getOne(new QueryWrapper<SysOrg>().lambda()
                    .eq(SysOrg::getId, teachingRoomId)
                    .eq(SysOrg::getEnable, true)
                    .eq(SysOrg::getType, OrgTypeEnum.TEACHING_ROOM)))) {
                throw ExceptionResultEnum.ERROR.exception("所选教研室不存在");
            }
            // 校验班级
            List<Long> clazzIdList = basicCourseParams.getClazzIdList();
            if (clazzIdList.size() != clazzIdList.stream().distinct().count()) {
                throw ExceptionResultEnum.ERROR.exception("输入的班级重复");
            }

            //基础数据组装
            Long orgId;
            BasicCourse basicCourse = new BasicCourse();
            basicCourse.setName(courseName);
            basicCourse.setCode(courseCode);
            basicCourse.setTeachingRoomId(teachingRoomId);
            basicCourse.setSchoolId(schoolId);

            if (!SystemConstant.longNotNull(id)) {
                // 没有id -》新增
                id = SystemConstant.getDbUuid();
                basicCourse.setId(id);
                basicCourse.setCreateId(sysUser.getId());
                basicCourse.setEnable(true);
                orgId = sysUser.getOrgId();
            } else {
                // 有id -》 更新
                BasicCourse old = this.getById(id);
                if (Objects.isNull(old) || old.getEnable().equals(false)) {
                    throw ExceptionResultEnum.ERROR.exception("课程主键不存在或被禁用");
                }
                orgId = old.getOrgId();
                basicCourse.setId(id);
                basicCourse.setUpdateId(sysUser.getId());
            }
            basicCourse.setOrgId(orgId);

            // 根据课程id和授课班级集合更新'basic_user_course'表
            basicUserCourseService.updateByCourseIdAndClazzList(id, clazzIdList, sysUser, orgId);

            basicCourseList.add(basicCourse);
        }
        return basicCourseList;
    }
}
